﻿using System;
using System.Runtime.InteropServices;
using MccDaq;//Project must have reference to MccDaq
using System.IO;

namespace TemperatureVoltageReadBackground
{
    class Program
    {
        public const int BLOCKSIZE = 50; //for best results use increments of packet size which is 32
        public const int CHANCOUNT = 4;
        public const int FIRSTCHANNEL = 0;
        public const int LASTCHANNEL = 3;
        public const int SAMPLERATE= 100;
        public const int BUFFERSIZE = BLOCKSIZE * CHANCOUNT;
        public const int HALFBUFFSIZE = BUFFERSIZE / 2;

        //this program works with both the USB-2408 & USB-2416. Uncomment the appropriate string
        public const string DEVICE = "USB-2416";
        //public const string DEVICE = "USB-2408";

        //external function not found in MccDaq to test for InstaCal setup. If you don't need to do this
        //then remove this and function call.
        //Used to determine InstaCal setup
        //************* Compile to x86 for 32 bit function call ****************************
        [DllImport("cbw32.dll")]
        public extern static long cbGetConfig([MarshalAs(UnmanagedType.I4)] int InfoType,
                                              [MarshalAs(UnmanagedType.I4)] int BoardNum,
                                              [MarshalAs(UnmanagedType.I4)] int DevNum,
                                              [MarshalAs(UnmanagedType.I4)] int ConfigItem,
                                              [MarshalAs(UnmanagedType.I8)] ref long ConfigVal
                                           );

        //**********************************************************************************
        public static StreamWriter fStream;

 
        static void Main(string[] args)
        {
            //Project must have reference to MccDaq
            MccDaq.ErrorInfo RetVal;
            int BoardNum = 0;
            int InfoType = 2;
            int DevNum = 0;
            int ConfigItem = 235; //Item number to get channel type
            long ConfigVal = 0;
            long ret;
            int Rate = SAMPLERATE;

            BoardNum = GetBoardNum(DEVICE);

            if (BoardNum == -1)
            {
                Console.WriteLine("No {0} detected!", DEVICE);
                WaitForKey();
                return;
            }
           
            MccBoard daq = new MccDaq.MccBoard(BoardNum);

            IntPtr buffer = MccService.ScaledWinBufAllocEx(BUFFERSIZE);

            if (buffer == IntPtr.Zero)
            {
                Console.WriteLine("Bad Memory Handle");
                WaitForKey();
                return;
            }

            short[] chType = new short[CHANCOUNT];
            short[] chArray = new short[CHANCOUNT];
            Range[] chRange = new Range[CHANCOUNT];

            //channels I want to read
            chArray[0] = 0;
            chArray[1] = 1;
            chArray[2] = 2;
            chArray[3] = 3;



            //Use InstaCal and set the first two channels to thermocouples
            //and the second two to voltage. Set the Data Rate to 1000
            //When using lengthy thermocouples set the channel Data Rate to 60Hz and reduce
            //the program's sample rate
            chType[0] = 2;//type for thermocouple
            chType[1] = 2;
            chType[2] = 5;//type for voltage
            chType[3] = 5;


            //Check to make sure the user ran InstaCal and set the channels appropriately
            for (DevNum = 0; DevNum < CHANCOUNT; DevNum++)
            {
                ret = cbGetConfig(InfoType, BoardNum, DevNum, ConfigItem, ref ConfigVal);
                if (ConfigVal != chType[DevNum])
                {
                    Console.Write("Configuration Error!\nRun InstaCal\n" +
                                    "\tset channels 0 & 1 to temperature\n" +
                                    "\tset channels 2 & 3 to voltage\n");
                    WaitForKey();
                    return;
                }
            }


            chRange[0] = Range.Bip10Volts;
            chRange[1] = Range.Bip10Volts;
            chRange[2] = Range.Bip10Volts;
            chRange[3] = Range.Bip10Volts;


            RetVal = daq.ALoadQueue(chArray, chRange, CHANCOUNT);
            IsError(RetVal);

            //ALoadQueue overrides channels and range in AInScan
            RetVal = daq.AInScan(   FIRSTCHANNEL,
                                    LASTCHANNEL,
                                    BUFFERSIZE,
                                    ref Rate,
                                    Range.Bip10Volts,
                                    buffer,
                                    ScanOptions.Background | ScanOptions.ScaleData | ScanOptions.Continuous
                                );
            IsError(RetVal);
            fStream = new StreamWriter(@"C:\Users\Public\Documents\DataFile1608G.asc");
            CreateFileHeaders(chArray); //writes basic info to the beginning of the file

            int Count = 0;
            int Index = 0;
            short daqStatus;
            bool ReadLower = true;
            double[] theArray = new double[BUFFERSIZE];

            System.ConsoleKeyInfo cki = new System.ConsoleKeyInfo();

            //Loop until key press
            do
            {
                RetVal = daq.GetStatus(out daqStatus, out Count, out Index, FunctionType.AiFunction);
                if ((Index >= HALFBUFFSIZE) & ReadLower) //check for 50% more data
                {
                    //get lower half of buffer - ScaledWinBufToArray returns engineering units
                    RetVal = MccService.ScaledWinBufToArray(buffer, theArray, 0, HALFBUFFSIZE);
                    IsError(RetVal);

                    DisplayData(theArray, HALFBUFFSIZE / CHANCOUNT);
                    ReadLower = false; //flag that controls the next read
                }
                else if ((Index < HALFBUFFSIZE) & !ReadLower)
                {
                    //get the upper half  - ScaledWinBufToArray returns engineering units
                    RetVal = MccService.ScaledWinBufToArray(buffer, theArray, HALFBUFFSIZE, HALFBUFFSIZE);
                    IsError(RetVal);

                    DisplayData(theArray, HALFBUFFSIZE / CHANCOUNT);
                    ReadLower = true;//flag that controls the next read
                }


            } while (!Console.KeyAvailable);

            cki = Console.ReadKey();

            //flush any buffered data out to disk
            fStream.Close();

            //stop the  acquisition
            RetVal = daq.StopBackground(FunctionType.AiFunction);

            //free up memory
            MccService.WinBufFreeEx(buffer);

            WaitForKey();

            
        }
        /************************************ Check Error *********************************************/
        public static int IsError(ErrorInfo e)
        {
            if (e.Value != 0)
            {
                if(e.Value == ErrorInfo.ErrorCode.BadRate )
                    Console.WriteLine("Bad Sample Rate! Run InstaCal and increase Data Rates or lower sample rate"); 
                else
                    Console.WriteLine(e.Message);
                WaitForKey();
                return 1;
            }
            return 0;
        }
        /************************************ Finds Device *******************************************/
        public static int GetBoardNum(string dev)
        {
            for (int BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                MccDaq.MccBoard daq = new MccDaq.MccBoard(BoardNum);
                if (daq.BoardName.Contains(dev))
                {
                    Console.WriteLine("{0} board number = {1}", dev, BoardNum.ToString());
                    daq.FlashLED();
                    return BoardNum;
                }
            }
            return -1;
        }
        /***************************** Waits for SpaceBar keypress **********************************/
        public static void WaitForKey()
        {
            Console.WriteLine("\nPress <SpaceBar> to continue...");

            System.ConsoleKeyInfo cki;
            do
            {
                cki = Console.ReadKey();
            } while (cki.Key != ConsoleKey.Spacebar);
        }
        /************************************ Displays Data *********************************************/
        public static void DisplayData(double[] datArray, int rows)
        {
            //Writes data to screen and to file
            int i = 0;

            for (int row = 0; row < rows; row++)
            {
                for (int c = 0; c < CHANCOUNT; c++)
                {
                    Console.Write("{0}\t", datArray[i].ToString("0.0000").PadLeft(10));
                    fStream.Write("{0}\t", datArray[i].ToString("0.0000"));
                    i++;
                }
                Console.Write("\r\n");
                fStream.Write("\r\n");
            }
            fStream.Flush();
        }
        /************************************ Create File Header *********************************************/

        public static void CreateFileHeaders(short[] x)
        {
            Console.WriteLine("This program reads channels {0} through {1}\n", FIRSTCHANNEL, LASTCHANNEL);

            //''''''''''''''''''''' create text file header strings ''''''''''''''''
            string[] hdr = new string[5];
            hdr[0] = "Recording date     : {0}";
            hdr[1] = "Block length       : {0}";
            hdr[2] = "Delta              : {0} sec.";
            hdr[3] = "Number of channels : {0}";

            //''''''''''''''''''''''' build column headers string '''''''''''''''''''

            for (int j = 0; j < CHANCOUNT; j++)
            {
                hdr[4] = hdr[4] + String.Format("In{0} [V]", x[j]);
                hdr[4] = hdr[4] + "\t";

            }
            hdr[4].Trim('\t');

            //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
            DateTime MyDate = DateTime.Now;

            fStream.WriteLine(hdr[0], MyDate);
            fStream.WriteLine(hdr[1], HALFBUFFSIZE / CHANCOUNT);
            fStream.WriteLine(hdr[2], (1 / SAMPLERATE));
            fStream.WriteLine(hdr[3], CHANCOUNT);
            fStream.WriteLine(hdr[4]);

            Console.WriteLine(hdr[0], MyDate);
            Console.WriteLine(hdr[1], HALFBUFFSIZE / CHANCOUNT);
            Console.WriteLine(hdr[2], (1 / SAMPLERATE));
            Console.WriteLine(hdr[3], CHANCOUNT);
            Console.WriteLine(hdr[4]);

        }

        /****************************************************************************************/
 
    }
}
